<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns="http://software.in2p3.fr/lavoisier/application.xsd"
                exclude-result-prefixes="cfg">
    <xsl:output method="xml" omit-xml-declaration="yes"/>
    <xsl:variable name="VERSION">8</xsl:variable>

    <xsl:template match="cfg:view[@path-template]">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates select="cfg:namespace | cfg:info"/>
            <xsl:call-template name="PATH_TEMPLATE">
                <xsl:with-param name="path" select="@path-template"/>
            </xsl:call-template>
            <xsl:apply-templates select="cfg:*[not(self::cfg:namespace | self::cfg:info)]"/>
        </xsl:copy>
    </xsl:template>
    <xsl:template match="cfg:view/@path-template"/>

    <xsl:template name="PATH_TEMPLATE">
        <xsl:param name="path"/>
        <xsl:if test="contains($path, '/{')">
            <xsl:variable name="remains" select="substring-after($path, '/{')"/>
            <xsl:variable name="current" select="substring-before($remains, '}')"/>
            <xsl:choose>
                <xsl:when test="contains($current, '=')">
                    <argument name="_{substring-before($current, '=')}">
                        <xsl:value-of select="substring-after($current, '=')"/>
                    </argument>
                </xsl:when>
                <xsl:otherwise>
                    <argument name="_{$current}"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:call-template name="PATH_TEMPLATE">
                <xsl:with-param name="path" select="$remains"/>
            </xsl:call-template>
        </xsl:if>
    </xsl:template>

    <xsl:template match="@eval | @disabled | @generate-match | @eval-authorized">
        <xsl:variable name="old" select="."/>
        <xsl:variable name="request">
            <xsl:call-template name="REPLACE">
                <xsl:with-param name="string" select="$old"/>
                <xsl:with-param name="search-start">request(</xsl:with-param><xsl:with-param name="search-end">)</xsl:with-param>
                <xsl:with-param name="replace-start">post(</xsl:with-param><xsl:with-param name="replace-end">)</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="view_request">
            <xsl:call-template name="REPLACE">
                <xsl:with-param name="string" select="$request"/>
                <xsl:with-param name="search-start">view_request(</xsl:with-param><xsl:with-param name="search-end">)</xsl:with-param>
                <xsl:with-param name="replace-start">view_post(</xsl:with-param><xsl:with-param name="replace-end">)</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="view">
            <xsl:choose>
                <xsl:when test="not(contains($view_request,'view(') and contains($view_request,','))">
                    <xsl:value-of select="$view_request"/>
                </xsl:when>
                <xsl:when test="contains($view_request,')/') and substring($view_request, string-length($view_request)) = ')'">
                    <xsl:call-template name="REPLACE">
                        <xsl:with-param name="string" select="$view_request"/>
                        <xsl:with-param name="search-start">view(</xsl:with-param><xsl:with-param name="search-end">,</xsl:with-param>
                        <xsl:with-param name="replace-start">view(concat(</xsl:with-param><xsl:with-param name="replace-end">,</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="contains($view_request,')/')">
                    <xsl:call-template name="REPLACE">
                        <xsl:with-param name="string" select="$view_request"/>
                        <xsl:with-param name="search-start">view(</xsl:with-param><xsl:with-param name="search-end">)/</xsl:with-param>
                        <xsl:with-param name="replace-start">view(concat(</xsl:with-param><xsl:with-param name="replace-end">))/</xsl:with-param>
                    </xsl:call-template>
                </xsl:when>
                <xsl:when test="substring($view_request, string-length($view_request)) = ')'">
                    <xsl:value-of select="substring-before($view_request,'view(')"/>
                    <xsl:text>view(concat(</xsl:text>
                    <xsl:value-of select="substring-after($view_request,'view(')"/>
                    <xsl:text>)</xsl:text>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$view_request"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:variable name="path_optional">
            <xsl:call-template name="REPLACE">
                <xsl:with-param name="string" select="$view"/>
                <xsl:with-param name="search-start">path(''</xsl:with-param><xsl:with-param name="search-end">)</xsl:with-param>
                <xsl:with-param name="replace-start">path(false()</xsl:with-param><xsl:with-param name="replace-end">)</xsl:with-param>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="path">
            <xsl:call-template name="REPLACE">
                <xsl:with-param name="string" select="$path_optional"/>
                <xsl:with-param name="search-start">path('</xsl:with-param><xsl:with-param name="search-end">')</xsl:with-param>
                <xsl:with-param name="replace-start">$_</xsl:with-param><xsl:with-param name="replace-end"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:attribute name="{local-name()}">
            <xsl:value-of select="$path"/>
        </xsl:attribute>
    </xsl:template>

    <xsl:template name="REPLACE">
        <xsl:param name="string"/>
        <xsl:param name="search-start"/><xsl:param name="search-end"/>
        <xsl:param name="replace-start"/><xsl:param name="replace-end"/>
        <xsl:choose>
            <xsl:when test="contains($string, $search-start) and contains(substring-after($string, $search-start), $search-end)">
                <xsl:variable name="remains" select="substring-after($string, $search-start)"/>
                <xsl:value-of select="substring-before($string, $search-start)"/><xsl:value-of select="$replace-start"/>
                <xsl:value-of select="substring-before($remains, $search-end)"/><xsl:value-of select="$replace-end"/>
                <xsl:call-template name="REPLACE">
                    <xsl:with-param name="string" select="substring-after($remains, $search-end)"/>
                    <xsl:with-param name="search-start" select="$search-start"/><xsl:with-param name="search-end" select="$search-end"/>
                    <xsl:with-param name="replace-start" select="$replace-start"/><xsl:with-param name="replace-end" select="$replace-end"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="$string"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <!-- common -->
    <xsl:template match="/cfg:config/@version">
        <xsl:attribute name="version">
            <xsl:value-of select="$VERSION"/>
        </xsl:attribute>
    </xsl:template>
    <xsl:template match="@* | comment()" priority="-1">
        <xsl:copy-of select="."/>
    </xsl:template>
    <xsl:template match="*" priority="-1">
        <xsl:copy><xsl:apply-templates select="@*"/>
            <xsl:apply-templates/>
        </xsl:copy>
    </xsl:template>
</xsl:stylesheet>